% Parameters 
h = 2;
Dt = 0.1;
% Slip = 0.5;%0.5;
AccErr = 0.05;
kmh2ms = 1000/3600;
a_max = 9.8;
a_min = 9.8;
Vmin = 80 * kmh2ms;
Vlim = 110 * kmh2ms;

StartDist = 45;

Unif = @(n) 2*rand(n,1) - 1;
sat = @(x) min(max(x,-a_min),a_max);

N = 300;
%% Generate car in front data.
acc_2 = zeros(N,1);
acc_2(1:100) = 0; %const speed at start.
acc_2(101:108) = -a_max;
acc_2(116:123) = a_max;
acc_2(131:138) = -a_max;
acc_2(146:153) = a_max;
acc_2(161:168) = -a_max;
acc_2(185:192) = a_max;
acc_2 = acc_2 + AccErr*Unif(N);
acc_2 = sat(acc_2);

v_2 = zeros(N,1);
p_2 = zeros(N,1);

v_2(1) = Vlim - 2*kmh2ms;
p_2(1) = StartDist;

for i=1:(N-1)
    p_2(i+1) = p_2(i) + Dt * v_2(i);
    v_2(i+1) = v_2(i) + Dt * acc_2(i);
end

%% Ego model
v_1 = zeros(N,1);
p_1 = zeros(N,1);
acc_1 = zeros(N,1);

v_1(1) = Vmin;
p_1(1) = 0;

%% Run Simulation
for i=1:N
    % Generate control signal
    acc_1(i) = (p_2(i) - p_1(i))/(h*Dt) - (1/h + 1/Dt)*v_1(i) + v_2(i)/h - 1;
    
    % Advance Car 1 Dynamics
    p_1(i+1) = p_1(i) + Dt * v_1(i);
    v_1(i+1) = v_1(i) + Dt * acc_1(i);
end
p_1 = p_1(1:end-1);
v_1 = v_1(1:end-1);

%% Plots
T = 0.1:0.1:(N/10);
figure;
plot(T, (p_2 - p_1)./v_1,'linewidth',2);
hold all;
grid on; xlabel('Time [sec]'); ylabel('Headway = {p_2 - p_1}/{v_1} [sec]');
print -depsc Headway_Paper

figure;
plot(T,p_2 - p_1,'linewidth',2); hold all;
grid on; xlabel('Time [sec]'); ylabel('Distance Between Cars [m]');
print -depsc Distance_Paper


figure;
% plot(T,v_1/kmh2ms,'linewidth',1); hold all;
plot(T,v_2/kmh2ms,'color','k','linewidth',2);
% legend('Ego Car','Other Car');
grid on; xlabel('Time [sec]'); ylabel('Velocity [km/h]');
print -depsc Velocity_Leader_Paper

figure;
plot(T,v_1/kmh2ms,'linewidth',2); hold all;
% plot(T,v_2/kmh2ms,'color','k','linewidth',2);
% legend('Ego Car','Other Car');
grid on; xlabel('Time [sec]'); ylabel('Velocity [km/h]');
print -depsc Velocity_Follower_Paper

figure;
% plot(T,acc_2,'color','k','linewidth',2);
plot(T,acc_1,'linewidth',2); hold all;
grid on; xlabel('Time [sec]'); ylabel('Acceleration [m/sec^2]');
print -depsc Acceleration_Follower_Paper

figure;
plot(T,acc_2,'color','k','linewidth',2);
% plot(T,acc_1,'linewidth',1); hold all;
grid on; xlabel('Time [sec]'); ylabel('Acceleration [m/sec^2]');
print -depsc Acceleration_Leader_Paper
